@@ -1,6 +1,7 @@
 /*
     GNOME Commander - A GNOME based file manager
     Copyright (C) 2001-2006 Marcus Bjurman
+    Copyright (C) 2007-2009 Piotr Eljasiak
 
     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -30,37 +31,28 @@
     GTK_CHECK_TYPE (obj, gnome_cmd_chown_component_get_type ())
 
 
-typedef struct _GnomeCmdChownComponent GnomeCmdChownComponent;
-typedef struct _GnomeCmdChownComponentPrivate GnomeCmdChownComponentPrivate;
-typedef struct _GnomeCmdChownComponentClass GnomeCmdChownComponentClass;
+struct GnomeCmdChownComponentPrivate;
 
 
-struct _GnomeCmdChownComponent
+struct GnomeCmdChownComponent
 {
     GtkTable parent;
+
     GnomeCmdChownComponentPrivate *priv;
 };
 
 
-struct _GnomeCmdChownComponentClass
+struct GnomeCmdChownComponentClass
 {
     GtkTableClass parent_class;
 };
 
 
-GtkWidget*
-gnome_cmd_chown_component_new (void);
+GtkWidget *gnome_cmd_chown_component_new ();
+GtkType gnome_cmd_chown_component_get_type ();
 
-GtkType
-gnome_cmd_chown_component_get_type (void);
-
-void
-gnome_cmd_chown_component_set (GnomeCmdChownComponent *comp, uid_t owner, gid_t group);
-
-uid_t
-gnome_cmd_chown_component_get_owner (GnomeCmdChownComponent *component);
-
-gid_t
-gnome_cmd_chown_component_get_group (GnomeCmdChownComponent *component);
+void gnome_cmd_chown_component_set (GnomeCmdChownComponent *comp, uid_t owner, gid_t group);
+uid_t gnome_cmd_chown_component_get_owner (GnomeCmdChownComponent *component);
+gid_t gnome_cmd_chown_component_get_group (GnomeCmdChownComponent *component);
 
 #endif // __GNOME_CMD_CHOWN_COMPONENT_H__
