@@ -1,6 +1,7 @@
 /*
     GNOME Commander - A GNOME based file manager
     Copyright (C) 2001-2006 Marcus Bjurman
+    Copyright (C) 2007-2009 Piotr Eljasiak
 
     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -21,6 +22,7 @@
 #include <sys/wait.h>
 #include <sys/types.h>
 #include <unistd.h>
+
 #include "gnome-cmd-includes.h"
 #include "gnome-cmd-cmdline.h"
 #include "gnome-cmd-file-selector.h"
@@ -36,6 +38,7 @@
 #include "gnome-cmd-con.h"
 #include "gnome-cmd-con-list.h"
 #include "gnome-cmd-bookmark-dialog.h"
+#include "owner.h"
 #include "utils.h"
 
 #include "../pixmaps/copy_file_names.xpm"
@@ -70,7 +73,7 @@ enum
 };
 
 
-struct _GnomeCmdMainWinPrivate
+struct GnomeCmdMainWinPrivate
 {
     FileSelectorID current_fs;
     GnomeCmdState state;
@@ -134,23 +137,23 @@ gint gnome_cmd_key_snooper(GtkWidget *grab_widget, GdkEventKey *event, GnomeCmdM
             (event->keyval == GDK_period)))
         return FALSE;
 
-    if (!gnome_cmd_data_get_alt_quick_search ())
+    if (!gnome_cmd_data.alt_quick_search)
         return FALSE;
 
     if (!state_is_alt (event->state) && !state_is_alt_shift (event->state))
         return FALSE;
 
     GnomeCmdFileSelector *fs = gnome_cmd_main_win_get_fs (mw, ACTIVE);
-    if (fs==NULL || fs->list==NULL)
+    if (fs==NULL || fs->file_list()==NULL)
         return FALSE;
 
-    if (!GTK_WIDGET_HAS_FOCUS (GTK_WIDGET (fs->list)))
+    if (!GTK_WIDGET_HAS_FOCUS (GTK_WIDGET (fs->file_list())))
         return FALSE;
 
-    if (gnome_cmd_file_list_quicksearch_shown (fs->list))
+    if (gnome_cmd_file_list_quicksearch_shown (fs->file_list()))
         return FALSE;
 
-    gnome_cmd_file_list_show_quicksearch (fs->list, event->keyval);
+    gnome_cmd_file_list_show_quicksearch (fs->file_list(), event->keyval);
 
     return TRUE;
 }
@@ -266,42 +269,48 @@ static void create_toolbar (GnomeCmdMainWin *mw, GnomeUIInfo *uiinfo)
 static void slide_set_100_0 (GtkMenu *menu, gpointer user_data)
 {
     gtk_paned_set_position (GTK_PANED (main_win->priv->paned),
-                            GTK_WIDGET (main_win)->allocation.width);
+                            gnome_cmd_data.list_orientation ? GTK_WIDGET (main_win)->allocation.height :
+                                                              GTK_WIDGET (main_win)->allocation.width);
 }
 
 
 static void slide_set_80_20 (GtkMenu *menu, gpointer user_data)
 {
     gtk_paned_set_position (GTK_PANED (main_win->priv->paned),
-                            (int)(GTK_WIDGET (main_win)->allocation.width*0.8f));
+                            gnome_cmd_data.list_orientation ? (int)(GTK_WIDGET (main_win)->allocation.height*0.8f) :
+                                                              (int)(GTK_WIDGET (main_win)->allocation.width*0.8f));
 }
 
 
 static void slide_set_60_40 (GtkMenu *menu, gpointer user_data)
 {
     gtk_paned_set_position (GTK_PANED (main_win->priv->paned),
-                            (int)(GTK_WIDGET (main_win)->allocation.width*0.6f));
+                            gnome_cmd_data.list_orientation ? (int)(GTK_WIDGET (main_win)->allocation.height*0.6f) :
+                                                              (int)(GTK_WIDGET (main_win)->allocation.width*0.6f));
 }
 
 
 static void slide_set_50_50 (GtkMenu *menu, gpointer user_data)
 {
     gtk_paned_set_position (GTK_PANED (main_win->priv->paned),
-                            (int)(GTK_WIDGET (main_win)->allocation.width*0.5f));
+                            gnome_cmd_data.list_orientation ? (int)(GTK_WIDGET (main_win)->allocation.height*0.5f) :
+                                                              (int)(GTK_WIDGET (main_win)->allocation.width*0.5f));
 }
 
 
 static void slide_set_40_60 (GtkMenu *menu, gpointer user_data)
 {
     gtk_paned_set_position (GTK_PANED (main_win->priv->paned),
-                            (int)(GTK_WIDGET (main_win)->allocation.width*0.4f));
+                            gnome_cmd_data.list_orientation ? (int)(GTK_WIDGET (main_win)->allocation.height*0.4f) :
+                                                              (int)(GTK_WIDGET (main_win)->allocation.width*0.4f));
 }
 
 
 static void slide_set_20_80 (GtkMenu *menu, gpointer user_data)
 {
     gtk_paned_set_position (GTK_PANED (main_win->priv->paned),
-                            (int)(GTK_WIDGET (main_win)->allocation.width*0.2f));
+                            gnome_cmd_data.list_orientation ? (int)(GTK_WIDGET (main_win)->allocation.height*0.2f) :
+                                                              (int)(GTK_WIDGET (main_win)->allocation.width*0.2f));
 }
 
 
@@ -333,7 +342,7 @@ static GtkWidget *create_slide_popup ()
 
     GtkWidget *menu = gtk_menu_new ();
     gtk_widget_ref (menu);
-    gtk_object_set_data_full (GTK_OBJECT (main_win), "slide-popup", menu, (GtkDestroyNotify)gtk_widget_unref);
+    gtk_object_set_data_full (GTK_OBJECT (main_win), "slide-popup", menu, (GtkDestroyNotify) gtk_widget_unref);
 
     // Fill the menu
 
@@ -450,13 +459,14 @@ static void create_buttonbar (GnomeCmdMainWin *mw)
 
 static gboolean on_slide_button_press (GtkWidget *widget, GdkEventButton *event, GnomeCmdMainWin *mw)
 {
-    if (event->type == GDK_BUTTON_PRESS && event->button == 3) {
+    if (event->type == GDK_BUTTON_PRESS && event->button == 3)
+    {
         GtkPaned *paned = GTK_PANED (mw->priv->paned);
 
         // Check that the handle was clicked and not one of the children
-        if (paned->handle == event->window) {
-            gtk_menu_popup (GTK_MENU(create_slide_popup ()), NULL, NULL, NULL, NULL,
-                            event->button, event->time);
+        if (paned->handle == event->window)
+        {
+            gtk_menu_popup (GTK_MENU(create_slide_popup ()), NULL, NULL, NULL, NULL, event->button, event->time);
             return TRUE;
         }
     }
@@ -466,22 +476,18 @@ static gboolean on_slide_button_press (GtkWidget *widget, GdkEventButton *event,
 
 static void on_main_win_realize (GtkWidget *widget, GnomeCmdMainWin *mw)
 {
-    gint w;
+    gnome_cmd_main_win_set_equal_panes (mw);
 
-    gdk_window_get_geometry (widget->window, NULL, NULL, &w, NULL, NULL);
-    gtk_paned_set_position (GTK_PANED (mw->priv->paned), w/2 - 5);
+    gnome_cmd_main_win_get_fs (mw, LEFT)->set_active(TRUE);
+    gnome_cmd_main_win_get_fs (mw, RIGHT)->set_active(FALSE);
+
+    // if (gnome_cmd_data.cmdline_visibility)
+    // {
+        // gchar *dpath = gnome_cmd_file_get_path (GNOME_CMD_FILE (gnome_cmd_main_win_get_fs (mw, LEFT)->get_directory()));
+        // gnome_cmd_cmdline_set_dir (GNOME_CMD_CMDLINE (mw->priv->cmdline), dpath);
+        // g_free (dpath);
+    // }
 
-    gnome_cmd_file_selector_set_active (gnome_cmd_main_win_get_fs (mw, LEFT), TRUE);
-    gnome_cmd_file_selector_set_active (gnome_cmd_main_win_get_fs (mw, RIGHT), FALSE);
-/*
-    if (gnome_cmd_data_get_cmdline_visibility ())
-    {
-        gchar *dpath = gnome_cmd_file_get_path (
-                GNOME_CMD_FILE (gnome_cmd_file_selector_get_directory (gnome_cmd_main_win_get_fs (mw, LEFT))));
-        gnome_cmd_cmdline_set_dir (GNOME_CMD_CMDLINE (mw->priv->cmdline), dpath);
-        g_free (dpath);
-    }
-*/
     gdk_window_set_icon (GTK_WIDGET (main_win)->window, NULL,
                          IMAGE_get_pixmap (PIXMAP_LOGO),
                          IMAGE_get_mask (PIXMAP_LOGO));
@@ -490,12 +496,12 @@ static void on_main_win_realize (GtkWidget *widget, GnomeCmdMainWin *mw)
 
 static gboolean on_left_fs_select (GtkCList *list, GdkEventButton *event, GnomeCmdMainWin *mw)
 {
-        mw->priv->current_fs = LEFT;
+    mw->priv->current_fs = LEFT;
 
-        gnome_cmd_file_selector_set_active (GNOME_CMD_FILE_SELECTOR (mw->priv->file_selector[LEFT]), TRUE);
-        gnome_cmd_file_selector_set_active (GNOME_CMD_FILE_SELECTOR (mw->priv->file_selector[RIGHT]), FALSE);
+    GNOME_CMD_FILE_SELECTOR (mw->priv->file_selector[LEFT])->set_active(TRUE);
+    GNOME_CMD_FILE_SELECTOR (mw->priv->file_selector[RIGHT])->set_active(FALSE);
 
-        return FALSE;
+    return FALSE;
 }
 
 
@@ -503,8 +509,8 @@ static gboolean on_right_fs_select (GtkCList *list, GdkEventButton *event, Gnome
 {
     mw->priv->current_fs = RIGHT;
 
-    gnome_cmd_file_selector_set_active (GNOME_CMD_FILE_SELECTOR (mw->priv->file_selector[RIGHT]), TRUE);
-    gnome_cmd_file_selector_set_active (GNOME_CMD_FILE_SELECTOR (mw->priv->file_selector[LEFT]), FALSE);
+    GNOME_CMD_FILE_SELECTOR (mw->priv->file_selector[RIGHT])->set_active(TRUE);
+    GNOME_CMD_FILE_SELECTOR (mw->priv->file_selector[LEFT])->set_active(FALSE);
 
     return FALSE;
 }
@@ -519,7 +525,7 @@ static void on_fs_list_resize_column (GtkCList *clist, gint column, gint width,
     if (!column_resize_lock)
     {
         column_resize_lock = TRUE;
-        gnome_cmd_data_set_fs_col_width ((guint)column, width);
+        gnome_cmd_data.fs_col_width[column] = width;
         gtk_clist_set_column_width (other_clist, column, width);
         column_resize_lock = FALSE;
     }
@@ -528,7 +534,7 @@ static void on_fs_list_resize_column (GtkCList *clist, gint column, gint width,
 
 static void on_size_allocate (GtkWidget *widget, GtkAllocation *allocation, gpointer user_data)
 {
-    switch (gnome_cmd_data_get_main_win_state())
+    switch (gnome_cmd_data.main_win_state)
     {
         case GDK_WINDOW_STATE_FULLSCREEN:
         case GDK_WINDOW_STATE_ICONIFIED:
@@ -536,7 +542,8 @@ static void on_size_allocate (GtkWidget *widget, GtkAllocation *allocation, gpoi
             break;
 
         default:
-            gnome_cmd_data_set_main_win_size (allocation->width, allocation->height);
+            gnome_cmd_data.main_win_width = allocation->width;
+            gnome_cmd_data.main_win_height = allocation->height;
     }
 }
 
@@ -548,12 +555,12 @@ inline void update_browse_buttons (GnomeCmdMainWin *mw, GnomeCmdFileSelector *fs
 
     if (fs == gnome_cmd_main_win_get_fs (mw, ACTIVE))
     {
-        if (gnome_cmd_data_get_toolbar_visibility ())
+        if (gnome_cmd_data.toolbar_visibility)
         {
-            gtk_widget_set_sensitive (mw->priv->tb_first_btn, gnome_cmd_file_selector_can_back (fs));
-            gtk_widget_set_sensitive (mw->priv->tb_back_btn, gnome_cmd_file_selector_can_back (fs));
-            gtk_widget_set_sensitive (mw->priv->tb_fwd_btn, gnome_cmd_file_selector_can_forward (fs));
-            gtk_widget_set_sensitive (mw->priv->tb_last_btn, gnome_cmd_file_selector_can_forward (fs));
+            gtk_widget_set_sensitive (mw->priv->tb_first_btn, fs->can_back());
+            gtk_widget_set_sensitive (mw->priv->tb_back_btn, fs->can_back());
+            gtk_widget_set_sensitive (mw->priv->tb_fwd_btn, fs->can_forward());
+            gtk_widget_set_sensitive (mw->priv->tb_last_btn, fs->can_forward());
         }
 
         gnome_cmd_main_menu_update_sens (GNOME_CMD_MAIN_MENU (mw->priv->menubar));
@@ -569,12 +576,12 @@ static void update_drop_con_button (GnomeCmdMainWin *mw, GnomeCmdFileSelector *f
     GnomeCmdPixmap *pm = NULL;
     static GtkWidget *prev_pixmap = NULL;
 
-    GnomeCmdCon *con = gnome_cmd_file_selector_get_connection (fs);
+    GnomeCmdCon *con = fs->get_connection();
     if (!con)
         return;
 
-    if (!gnome_cmd_data_get_toolbar_visibility ()
-        || (gnome_cmd_data_get_skip_mounting () && GNOME_CMD_IS_CON_DEVICE (con)))
+    if (!gnome_cmd_data.toolbar_visibility
+        || (gnome_cmd_data.skip_mounting && GNOME_CMD_IS_CON_DEVICE (con)))
         return;
 
     GtkWidget *btn = mw->priv->tb_con_drop_btn;
@@ -632,16 +639,17 @@ static void on_fs_dir_change (GnomeCmdFileSelector *fs, const gchar dir, GnomeCm
 
 static void restore_size_and_pos (GnomeCmdMainWin *mw)
 {
-    gint w, h, x, y;
+    gint x, y;
 
-    gnome_cmd_data_get_main_win_size (&w, &h);
-    gtk_widget_set_size_request (GTK_WIDGET (main_win), w, h);
+    gtk_widget_set_size_request (GTK_WIDGET (main_win),
+                                 gnome_cmd_data.main_win_width,
+                                 gnome_cmd_data.main_win_height);
 
     gnome_cmd_data_get_main_win_pos (&x, &y);
     if (x >= 0 && y >= 0)
         gtk_window_move (GTK_WINDOW (mw), x, y);
 
-    switch (gnome_cmd_data_get_main_win_state ())
+    switch (gnome_cmd_data.main_win_state)
     {
         case GDK_WINDOW_STATE_MAXIMIZED:
         case GDK_WINDOW_STATE_FULLSCREEN:
@@ -671,8 +679,8 @@ static gboolean on_window_state_event (GtkWidget *mw, GdkEventWindowState *event
                 break;
 
         case GDK_WINDOW_STATE_ICONIFIED:
-            if (gnome_cmd_data_get_main_win_state() == GDK_WINDOW_STATE_MAXIMIZED ||  // prev state
-                gnome_cmd_data_get_main_win_state() == GDK_WINDOW_STATE_FULLSCREEN)
+            if (gnome_cmd_data.main_win_state == GDK_WINDOW_STATE_MAXIMIZED ||  // prev state
+                gnome_cmd_data.main_win_state == GDK_WINDOW_STATE_FULLSCREEN)
                 break;  // not usable
 
         default:            // other are usable
@@ -680,7 +688,7 @@ static gboolean on_window_state_event (GtkWidget *mw, GdkEventWindowState *event
             gnome_cmd_data_set_main_win_pos (x, y);
     }
 
-    gnome_cmd_data_set_main_win_state (event->new_window_state);
+    gnome_cmd_data.main_win_state = event->new_window_state;
 
     return FALSE;
 }
@@ -700,14 +708,17 @@ static void destroy (GtkObject *object)
         main_win->priv->key_snooper_id = 0;
     }
 
-    GnomeCmdDir *dir = gnome_cmd_file_selector_get_directory (gnome_cmd_main_win_get_fs (main_win, LEFT));
+    GnomeCmdDir *dir = gnome_cmd_main_win_get_fs (main_win, LEFT)->get_directory();
     if (con_home == gnome_cmd_dir_get_connection (dir))
         gnome_cmd_data_set_start_dir (LEFT, gnome_cmd_file_get_path (GNOME_CMD_FILE (dir)));
 
-    dir = gnome_cmd_file_selector_get_directory (gnome_cmd_main_win_get_fs (main_win, RIGHT));
+    dir = gnome_cmd_main_win_get_fs (main_win, RIGHT)->get_directory();
     if (con_home == gnome_cmd_dir_get_connection (dir))
         gnome_cmd_data_set_start_dir (RIGHT, gnome_cmd_file_get_path (GNOME_CMD_FILE (dir)));
 
+    if (main_win->advrename_dlg)
+        gtk_widget_destroy (*main_win->advrename_dlg);
+
     gtk_main_quit ();
 }
 
@@ -727,7 +738,7 @@ static void class_init (GnomeCmdMainWinClass *klass)
     parent_class = (GnomeAppClass *) gtk_type_class (gnome_app_get_type ());
 
     main_win_signals[SWITCH_FS] =
-        gtk_signal_new ("switch_fs",
+        gtk_signal_new ("switch-fs",
             GTK_RUN_LAST,
             G_OBJECT_CLASS_TYPE (object_class),
             GTK_SIGNAL_OFFSET (GnomeCmdMainWinClass, switch_fs),
@@ -740,7 +751,6 @@ static void class_init (GnomeCmdMainWinClass *klass)
     klass->switch_fs = gnome_cmd_main_win_real_switch_fs;
 }
 
-
 static void init (GnomeCmdMainWin *mw)
 {
     /* It is very important that this global variable gets assigned here so that
@@ -748,6 +758,7 @@ static void init (GnomeCmdMainWin *mw)
      */
     main_win = GNOME_CMD_MAIN_WIN (mw);
 
+    mw->advrename_dlg = NULL;
     mw->priv = g_new0 (GnomeCmdMainWinPrivate, 1);
     mw->priv->current_fs = LEFT;
     mw->priv->accel_group = gtk_accel_group_new ();
@@ -761,8 +772,8 @@ static void init (GnomeCmdMainWin *mw)
     mw->priv->file_selector[LEFT] = NULL;
     mw->priv->file_selector[RIGHT] = NULL;
 
-    gnome_app_construct (GNOME_APP (main_win), "gnome-commander", geteuid() ? _("GNOME Commander") :
-                                                                              _("GNOME Commander - ROOT PRIVILEGES"));
+    gnome_app_construct (GNOME_APP (main_win), "gnome-commander", gcmd_owner.is_root() ? _("GNOME Commander - ROOT PRIVILEGES") :
+                                                                                         _("GNOME Commander"));
     gtk_object_set_data (GTK_OBJECT (main_win), "main_win", main_win);
     restore_size_and_pos (mw);
     gtk_window_set_policy (GTK_WINDOW (main_win), TRUE, TRUE, FALSE);
@@ -784,7 +795,7 @@ static void init (GnomeCmdMainWin *mw)
 
     gnome_app_set_contents (GNOME_APP (main_win), mw->priv->vbox);
 
-    mw->priv->paned = gnome_cmd_data_get_list_orientation () ? gtk_vpaned_new () : gtk_hpaned_new ();
+    mw->priv->paned = gnome_cmd_data.list_orientation ? gtk_vpaned_new () : gtk_hpaned_new ();
 
     gtk_widget_ref (mw->priv->paned);
     gtk_object_set_data_full (GTK_OBJECT (main_win), "paned", mw->priv->paned,
@@ -817,43 +828,38 @@ static void init (GnomeCmdMainWin *mw)
     gtk_signal_connect (GTK_OBJECT (mw->priv->file_selector[LEFT]), "changed-dir", GTK_SIGNAL_FUNC (on_fs_dir_change), mw);
     gtk_signal_connect (GTK_OBJECT (mw->priv->file_selector[RIGHT]), "changed-dir", GTK_SIGNAL_FUNC (on_fs_dir_change), mw);
 
-    gtk_signal_connect (GTK_OBJECT (GNOME_CMD_FILE_SELECTOR (mw->priv->file_selector[LEFT])->list),
-                        "resize_column", GTK_SIGNAL_FUNC (on_fs_list_resize_column),
-                        GNOME_CMD_FILE_SELECTOR (mw->priv->file_selector[RIGHT])->list);
-    gtk_signal_connect (GTK_OBJECT (GNOME_CMD_FILE_SELECTOR (mw->priv->file_selector[RIGHT])->list),
-                        "resize_column", GTK_SIGNAL_FUNC (on_fs_list_resize_column),
-                        GNOME_CMD_FILE_SELECTOR (mw->priv->file_selector[LEFT])->list);
-    gtk_signal_connect (GTK_OBJECT (GNOME_CMD_FILE_SELECTOR (mw->priv->file_selector[LEFT])->list),
-                        "button_press_event", GTK_SIGNAL_FUNC (on_left_fs_select), mw);
-    gtk_signal_connect (GTK_OBJECT (GNOME_CMD_FILE_SELECTOR (mw->priv->file_selector[RIGHT])->list),
-                        "button_press_event", GTK_SIGNAL_FUNC (on_right_fs_select), mw);
-    gtk_signal_connect (GTK_OBJECT (mw), "size-allocate",
-                        GTK_SIGNAL_FUNC (on_size_allocate), mw);
-    gtk_signal_connect (GTK_OBJECT (mw), "delete-event",
-                        GTK_SIGNAL_FUNC (on_delete_event), mw);
-    gtk_signal_connect (GTK_OBJECT (mw->priv->paned),
-                        "button_press_event", GTK_SIGNAL_FUNC (on_slide_button_press), mw);
+    gtk_signal_connect (GTK_OBJECT (GNOME_CMD_FILE_SELECTOR (mw->priv->file_selector[LEFT])->file_list()),
+                        "resize-column", GTK_SIGNAL_FUNC (on_fs_list_resize_column),
+                        GNOME_CMD_FILE_SELECTOR (mw->priv->file_selector[RIGHT])->file_list());
+    gtk_signal_connect (GTK_OBJECT (GNOME_CMD_FILE_SELECTOR (mw->priv->file_selector[RIGHT])->file_list()),
+                        "resize-column", GTK_SIGNAL_FUNC (on_fs_list_resize_column),
+                        GNOME_CMD_FILE_SELECTOR (mw->priv->file_selector[LEFT])->file_list());
+    gtk_signal_connect (GTK_OBJECT (GNOME_CMD_FILE_SELECTOR (mw->priv->file_selector[LEFT])->file_list()),
+                        "button-press-event", GTK_SIGNAL_FUNC (on_left_fs_select), mw);
+    gtk_signal_connect (GTK_OBJECT (GNOME_CMD_FILE_SELECTOR (mw->priv->file_selector[RIGHT])->file_list()),
+                        "button-press-event", GTK_SIGNAL_FUNC (on_right_fs_select), mw);
+    gtk_signal_connect (GTK_OBJECT (mw), "size-allocate", GTK_SIGNAL_FUNC (on_size_allocate), mw);
+    gtk_signal_connect (GTK_OBJECT (mw), "delete-event", GTK_SIGNAL_FUNC (on_delete_event), mw);
+    gtk_signal_connect (GTK_OBJECT (mw->priv->paned), "button-press-event", GTK_SIGNAL_FUNC (on_slide_button_press), mw);
     g_signal_connect (mw, "window-state-event", GTK_SIGNAL_FUNC (on_window_state_event), NULL);
 
-    gnome_cmd_file_selector_update_connections (gnome_cmd_main_win_get_fs (mw, LEFT));
-    gnome_cmd_file_selector_update_connections (gnome_cmd_main_win_get_fs (mw, RIGHT));
+    gnome_cmd_main_win_get_fs (mw, LEFT)->update_connections();
+    gnome_cmd_main_win_get_fs (mw, RIGHT)->update_connections();
 
-    gnome_cmd_file_selector_set_connection (gnome_cmd_main_win_get_fs (mw, LEFT), get_home_con ());
-    gnome_cmd_file_selector_set_connection (gnome_cmd_main_win_get_fs (mw, RIGHT), get_home_con ());
+    gnome_cmd_main_win_get_fs (mw, LEFT)->set_connection(get_home_con ());
+    gnome_cmd_main_win_get_fs (mw, RIGHT)->set_connection(get_home_con ());
 
-    gnome_cmd_file_selector_goto_directory (gnome_cmd_main_win_get_fs (mw, LEFT),
-                                            start_dir_left ? start_dir_left : gnome_cmd_data_get_start_dir (LEFT));
-    gnome_cmd_file_selector_goto_directory (gnome_cmd_main_win_get_fs (mw, RIGHT),
-                                            start_dir_right ? start_dir_right : gnome_cmd_data_get_start_dir (RIGHT));
+    gnome_cmd_main_win_get_fs (mw, LEFT)->goto_directory(start_dir_left ? start_dir_left : gnome_cmd_data_get_start_dir (LEFT));
+    gnome_cmd_main_win_get_fs (mw, RIGHT)->goto_directory(start_dir_right ? start_dir_right : gnome_cmd_data_get_start_dir (RIGHT));
 
     gtk_window_add_accel_group (GTK_WINDOW (main_win), mw->priv->accel_group);
     gnome_cmd_main_win_focus_file_lists (main_win);
 
-    mw->priv->key_snooper_id = gtk_key_snooper_install((GtkKeySnoopFunc)gnome_cmd_key_snooper,(gpointer)mw);
+    mw->priv->key_snooper_id = gtk_key_snooper_install ((GtkKeySnoopFunc) gnome_cmd_key_snooper, (gpointer) mw);
 }
 
 
-GtkType gnome_cmd_main_win_get_type (void)
+GtkType gnome_cmd_main_win_get_type ()
 {
     static GtkType mw_type = 0;
 
@@ -918,17 +924,17 @@ void gnome_cmd_main_win_update_style (GnomeCmdMainWin *mw)
 
     IMAGE_clear_mime_cache ();
 
-    gnome_cmd_file_selector_update_style (gnome_cmd_main_win_get_fs (mw, LEFT));
-    gnome_cmd_file_selector_update_style (gnome_cmd_main_win_get_fs (mw, RIGHT));
+    gnome_cmd_main_win_get_fs (mw, LEFT)->update_style();
+    gnome_cmd_main_win_get_fs (mw, RIGHT)->update_style();
 
-    if (gnome_cmd_data_get_cmdline_visibility ())
+    if (gnome_cmd_data.cmdline_visibility)
         gnome_cmd_cmdline_update_style (GNOME_CMD_CMDLINE (mw->priv->cmdline));
 }
 
 
 void gnome_cmd_main_win_focus_cmdline (GnomeCmdMainWin *mw)
 {
-    if (gnome_cmd_data_get_cmdline_visibility ())
+    if (gnome_cmd_data.cmdline_visibility)
     {
         gnome_cmd_cmdline_focus (GNOME_CMD_CMDLINE (mw->priv->cmdline));
         mw->priv->focused_widget = mw->priv->cmdline;
@@ -938,8 +944,8 @@ void gnome_cmd_main_win_focus_cmdline (GnomeCmdMainWin *mw)
 
 void gnome_cmd_main_win_focus_file_lists (GnomeCmdMainWin *mw)
 {
-    gnome_cmd_file_selector_set_active (gnome_cmd_main_win_get_fs (mw, ACTIVE), TRUE);
-    gnome_cmd_file_selector_set_active (gnome_cmd_main_win_get_fs (mw, INACTIVE), FALSE);
+    gnome_cmd_main_win_get_fs (mw, ACTIVE)->set_active(TRUE);
+    gnome_cmd_main_win_get_fs (mw, INACTIVE)->set_active(FALSE);
 
     mw->priv->focused_widget = mw->priv->file_selector[mw->priv->current_fs];
 }
@@ -975,7 +981,7 @@ gboolean gnome_cmd_main_win_keypressed (GnomeCmdMainWin *mw, GdkEventKey *event)
                 return TRUE;
 
             case GDK_F8:
-                if (gnome_cmd_data_get_cmdline_visibility ())
+                if (gnome_cmd_data.cmdline_visibility)
                     gnome_cmd_cmdline_show_history (GNOME_CMD_CMDLINE (mw->priv->cmdline));
                 return TRUE;
         }
@@ -986,10 +992,10 @@ gboolean gnome_cmd_main_win_keypressed (GnomeCmdMainWin *mw, GdkEventKey *event)
         {
             case GDK_H:
             case GDK_h:
-                gnome_cmd_data_set_hidden_filter (!gnome_cmd_data_get_hidden_filter ());
+                gnome_cmd_data.filter_settings.hidden = !gnome_cmd_data.filter_settings.hidden;
                 gnome_cmd_style_create ();
                 gnome_cmd_main_win_update_style (main_win);
-                gnome_cmd_data_save ();
+                gnome_cmd_data.save();
                 return TRUE;
         }
     }
@@ -997,22 +1003,24 @@ gboolean gnome_cmd_main_win_keypressed (GnomeCmdMainWin *mw, GdkEventKey *event)
     {
         switch (event->keyval)
         {
+            case GDK_e:
+            case GDK_E:
             case GDK_Down:
-                if (gnome_cmd_data_get_cmdline_visibility ())
+                if (gnome_cmd_data.cmdline_visibility)
                     gnome_cmd_cmdline_show_history (GNOME_CMD_CMDLINE (mw->priv->cmdline));
                 return TRUE;
 
             case GDK_u:
             case GDK_U:
                 {
-                    GnomeCmdDir *dir1 = gnome_cmd_file_selector_get_directory (gnome_cmd_main_win_get_fs (mw, LEFT));
-                    GnomeCmdDir *dir2 = gnome_cmd_file_selector_get_directory (gnome_cmd_main_win_get_fs (mw, RIGHT));
+                    GnomeCmdDir *dir1 = gnome_cmd_main_win_get_fs (mw, LEFT)->get_directory();
+                    GnomeCmdDir *dir2 = gnome_cmd_main_win_get_fs (mw, RIGHT)->get_directory();
 
                     gnome_cmd_dir_ref (dir1);
                     gnome_cmd_dir_ref (dir2);
 
-                    gnome_cmd_file_selector_set_directory (gnome_cmd_main_win_get_fs (mw, LEFT), dir2);
-                    gnome_cmd_file_selector_set_directory (gnome_cmd_main_win_get_fs (mw, RIGHT), dir1);
+                    gnome_cmd_main_win_get_fs (mw, LEFT)->set_directory(dir2);
+                    gnome_cmd_main_win_get_fs (mw, RIGHT)->set_directory(dir1);
 
                     gnome_cmd_dir_unref (dir1);
                     gnome_cmd_dir_unref (dir2);
@@ -1036,7 +1044,7 @@ gboolean gnome_cmd_main_win_keypressed (GnomeCmdMainWin *mw, GdkEventKey *event)
             {
                 GnomeCmdConFtp *con = GNOME_CMD_CON_FTP (gnome_cmd_con_list_get_all_ftp (gnome_cmd_con_list_get ())->data);
 
-                gnome_cmd_file_selector_set_connection (gnome_cmd_main_win_get_fs (main_win, ACTIVE), GNOME_CMD_CON (con));
+                gnome_cmd_main_win_get_fs (main_win, ACTIVE)->set_connection(GNOME_CMD_CON (con));
             }
             break;
         }
@@ -1092,7 +1100,7 @@ gboolean gnome_cmd_main_win_keypressed (GnomeCmdMainWin *mw, GdkEventKey *event)
                     return TRUE;
             }
 
-    if (gnome_cmd_file_selector_keypressed (gnome_cmd_main_win_get_fs (mw, ACTIVE), event))
+    if (gnome_cmd_main_win_get_fs (mw, ACTIVE)->key_pressed(event))
         return TRUE;
 
     return FALSE;
@@ -1108,11 +1116,6 @@ void gnome_cmd_main_win_switch_fs (GnomeCmdMainWin *mw, GnomeCmdFileSelector *fs
 }
 
 
-gboolean gnome_cmd_main_win_is_fs_active (GnomeCmdMainWin *mw, FileSelectorID fs)
-{
-    return mw->priv->current_fs == fs;
-}
-
 static void gnome_cmd_main_win_real_switch_fs (GnomeCmdMainWin *mw, GnomeCmdFileSelector *fs)
 {
     g_return_if_fail (GNOME_CMD_IS_MAIN_WIN (mw));
@@ -1122,8 +1125,8 @@ static void gnome_cmd_main_win_real_switch_fs (GnomeCmdMainWin *mw, GnomeCmdFile
         return;
 
     mw->priv->current_fs = (FileSelectorID) !mw->priv->current_fs;
-    gnome_cmd_file_selector_set_active (gnome_cmd_main_win_get_fs (mw, ACTIVE), TRUE);
-    gnome_cmd_file_selector_set_active (gnome_cmd_main_win_get_fs (mw, INACTIVE), FALSE);
+    gnome_cmd_main_win_get_fs (mw, ACTIVE)->set_active(TRUE);
+    gnome_cmd_main_win_get_fs (mw, INACTIVE)->set_active(FALSE);
 
     update_browse_buttons (mw, fs);
     update_drop_con_button (mw, fs);
@@ -1174,7 +1177,7 @@ void gnome_cmd_main_win_update_toolbar_visibility (GnomeCmdMainWin *mw)
         GNOMEUIINFO_END
     };
 
-    if (gnome_cmd_data_get_toolbar_visibility ())
+    if (gnome_cmd_data.toolbar_visibility)
     {
         create_toolbar (mw, toolbar_uiinfo);
         gtk_box_pack_start (GTK_BOX (mw->priv->vbox), mw->priv->toolbar, FALSE, TRUE, 0);
@@ -1200,7 +1203,7 @@ void gnome_cmd_main_win_update_buttonbar_visibility (GnomeCmdMainWin *mw)
 {
     g_return_if_fail (GNOME_CMD_IS_MAIN_WIN (mw));
 
-    if (gnome_cmd_data_get_buttonbar_visibility ())
+    if (gnome_cmd_data.buttonbar_visibility)
     {
         create_buttonbar (mw);
         gtk_box_pack_start (GTK_BOX (mw->priv->vbox), mw->priv->buttonbar_sep, FALSE, TRUE, 0);
@@ -1222,7 +1225,7 @@ void gnome_cmd_main_win_update_cmdline_visibility (GnomeCmdMainWin *mw)
 {
     g_return_if_fail (GNOME_CMD_IS_MAIN_WIN (mw));
 
-    if (gnome_cmd_data_get_cmdline_visibility ())
+    if (gnome_cmd_data.cmdline_visibility)
     {
         gint pos = 3;
         mw->priv->cmdline_sep = create_separator (FALSE);
@@ -1231,7 +1234,7 @@ void gnome_cmd_main_win_update_cmdline_visibility (GnomeCmdMainWin *mw)
         gtk_object_set_data_full (GTK_OBJECT (main_win), "cmdline", mw->priv->cmdline,
                                   (GtkDestroyNotify) gtk_widget_unref);
         gtk_widget_show (mw->priv->cmdline);
-        if (gnome_cmd_data_get_toolbar_visibility ())
+        if (gnome_cmd_data.toolbar_visibility)
             pos += 2;
         gtk_box_pack_start (GTK_BOX (mw->priv->vbox), mw->priv->cmdline_sep, FALSE, TRUE, 0);
         gtk_box_pack_start (GTK_BOX (mw->priv->vbox), mw->priv->cmdline, FALSE, TRUE, 1);
@@ -1250,22 +1253,12 @@ void gnome_cmd_main_win_update_cmdline_visibility (GnomeCmdMainWin *mw)
 }
 
 
-void gnome_cmd_main_win_update_connections (GnomeCmdMainWin *mw)
-{
-    g_return_if_fail (GNOME_CMD_IS_MAIN_WIN (mw));
-
-    gnome_cmd_file_selector_update_connections (gnome_cmd_main_win_get_fs (mw, LEFT));
-    gnome_cmd_file_selector_update_connections (gnome_cmd_main_win_get_fs (mw, RIGHT));
-    gnome_cmd_main_menu_update_connections (GNOME_CMD_MAIN_MENU (mw->priv->menubar));
-}
-
-
 void gnome_cmd_main_win_update_list_orientation (GnomeCmdMainWin *mw)
 {
-    gint pos = 2;
-
     g_return_if_fail (GNOME_CMD_IS_MAIN_WIN (mw));
 
+    gint pos = 2;
+
     gtk_widget_ref (mw->priv->file_selector[LEFT]);
     gtk_widget_ref (mw->priv->file_selector[RIGHT]);
     gtk_container_remove (GTK_CONTAINER (mw->priv->paned), mw->priv->file_selector[LEFT]);
@@ -1273,7 +1266,7 @@ void gnome_cmd_main_win_update_list_orientation (GnomeCmdMainWin *mw)
 
     gtk_object_destroy (GTK_OBJECT (mw->priv->paned));
 
-    mw->priv->paned = gnome_cmd_data_get_list_orientation () ? gtk_vpaned_new () : gtk_hpaned_new ();
+    mw->priv->paned = gnome_cmd_data.list_orientation ? gtk_vpaned_new () : gtk_hpaned_new ();
 
     gtk_widget_ref (mw->priv->paned);
     gtk_object_set_data_full (GTK_OBJECT (main_win), "paned", mw->priv->paned, (GtkDestroyNotify) gtk_widget_unref);
@@ -1282,7 +1275,7 @@ void gnome_cmd_main_win_update_list_orientation (GnomeCmdMainWin *mw)
     gtk_paned_pack1 (GTK_PANED (mw->priv->paned), mw->priv->file_selector[LEFT], TRUE, TRUE);
     gtk_paned_pack2 (GTK_PANED (mw->priv->paned), mw->priv->file_selector[RIGHT], TRUE, TRUE);
 
-    if (gnome_cmd_data_get_toolbar_visibility ())
+    if (gnome_cmd_data.toolbar_visibility)
         pos += 2;
 
     gtk_box_pack_start (GTK_BOX (mw->priv->vbox), mw->priv->paned, TRUE, TRUE, 0);
@@ -1291,7 +1284,7 @@ void gnome_cmd_main_win_update_list_orientation (GnomeCmdMainWin *mw)
     gtk_widget_unref (mw->priv->file_selector[LEFT]);
     gtk_widget_unref (mw->priv->file_selector[RIGHT]);
 
-    gtk_signal_connect (GTK_OBJECT (mw->priv->paned), "button_press_event", GTK_SIGNAL_FUNC (on_slide_button_press), mw);
+    gtk_signal_connect (GTK_OBJECT (mw->priv->paned), "button-press-event", GTK_SIGNAL_FUNC (on_slide_button_press), mw);
     slide_set_50_50 (NULL, NULL);
 }
 
@@ -1310,16 +1303,16 @@ GnomeCmdState *gnome_cmd_main_win_get_state (GnomeCmdMainWin *mw)
 
     GnomeCmdFileSelector *fs1 = gnome_cmd_main_win_get_fs (main_win, ACTIVE);
     GnomeCmdFileSelector *fs2 = gnome_cmd_main_win_get_fs (main_win, INACTIVE);
-    GnomeCmdDir *dir1 = gnome_cmd_file_selector_get_directory (fs1);
-    GnomeCmdDir *dir2 = gnome_cmd_file_selector_get_directory (fs2);
+    GnomeCmdDir *dir1 = fs1->get_directory();
+    GnomeCmdDir *dir2 = fs2->get_directory();
 
     GnomeCmdState *state = &mw->priv->state;
     state->active_dir_uri = gnome_cmd_file_get_uri (GNOME_CMD_FILE (dir1));
     state->inactive_dir_uri = gnome_cmd_file_get_uri (GNOME_CMD_FILE (dir2));
-    state->active_dir_files = gnome_cmd_file_list_get_all_files (fs1->list);
-    state->inactive_dir_files = gnome_cmd_file_list_get_all_files (fs2->list);
-    state->active_dir_selected_files = gnome_cmd_file_list_get_selected_files (fs1->list);
-    state->inactive_dir_selected_files = gnome_cmd_file_list_get_selected_files (fs2->list);
+    state->active_dir_files = fs1->file_list()->get_visible_files();
+    state->inactive_dir_files = fs2->file_list()->get_visible_files();
+    state->active_dir_selected_files = fs1->file_list()->get_selected_files();
+    state->inactive_dir_selected_files = fs2->file_list()->get_selected_files();
 
     return state;
 }
