@@ -1,6 +1,7 @@
 /*
     GNOME Commander - A GNOME based file manager
     Copyright (C) 2001-2006 Marcus Bjurman
+    Copyright (C) 2007-2009 Piotr Eljasiak
 
     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -33,11 +34,13 @@
 #include "gnome-cmd-con-dialog.h"
 #include "gnome-cmd-remote-dialog.h"
 #include "gnome-cmd-main-win.h"
+#include "gnome-cmd-mkdir-dialog.h"
 #include "gnome-cmd-options-dialog.h"
 #include "gnome-cmd-prepare-copy-dialog.h"
 #include "gnome-cmd-prepare-move-dialog.h"
 #include "gnome-cmd-python-plugin.h"
 #include "gnome-cmd-search-dialog.h"
+#include "gnome-cmd-advrename-dialog.h"
 #include "gnome-cmd-key-shortcuts-dialog.h"
 #include "gnome-cmd-user-actions.h"
 #include "plugin_manager.h"
@@ -57,19 +60,19 @@ inline GnomeCmdFileList *get_fl (const FileSelectorID fsID)
 {
     GnomeCmdFileSelector *fs = get_fs (fsID);
 
-    return fs ? fs->list : NULL;
+    return fs ? fs->file_list() : NULL;
 }
 
 
 // The file returned from this function is not to be unrefed
 inline GnomeCmdFile *get_selected_file (const FileSelectorID fsID)
 {
-    GnomeCmdFile *finfo = gnome_cmd_file_list_get_first_selected_file (get_fl (fsID));
+    GnomeCmdFile *f = get_fl (fsID)->get_first_selected_file();
 
-    if (!finfo)
+    if (!f)
         create_error_dialog (_("No file selected"));
 
-    return finfo;
+    return f;
 }
 
 
@@ -82,12 +85,12 @@ inline gboolean append_real_path (string &s, const gchar *name)
 }
 
 
-inline gboolean append_real_path (string &s, GnomeCmdFile *finfo)
+inline gboolean append_real_path (string &s, GnomeCmdFile *f)
 {
-    if (!finfo)
+    if (!f)
         return FALSE;
 
-    gchar *name = g_shell_quote (gnome_cmd_file_get_real_path (finfo));
+    gchar *name = g_shell_quote (gnome_cmd_file_get_real_path (f));
 
     append_real_path (s, name);
 
@@ -108,7 +111,7 @@ inline bool operator < (const GdkEventKey &e1, const GdkEventKey &e2)
     if (e1.keyval > e2.keyval)
         return false;
 
-#ifdef HAVE_GTK_2_10
+#if GTK_CHECK_VERSION (2, 10, 0)
     return (e1.state & (GDK_SHIFT_MASK | GDK_CONTROL_MASK | GDK_MOD1_MASK | GDK_SUPER_MASK | GDK_HYPER_MASK | GDK_META_MASK)) < (e2.state & (GDK_SHIFT_MASK | GDK_CONTROL_MASK | GDK_MOD1_MASK | GDK_SUPER_MASK | GDK_HYPER_MASK | GDK_META_MASK));
 #else
     return (e1.state & (GDK_SHIFT_MASK | GDK_CONTROL_MASK | GDK_MOD1_MASK | GDK_MOD4_MASK)) < (e2.state & (GDK_SHIFT_MASK | GDK_CONTROL_MASK | GDK_MOD1_MASK | GDK_MOD4_MASK));
@@ -132,6 +135,7 @@ static UserActionData user_actions_data[] = {
                                              {bookmarks_add_current, "bookmarks.add_current", N_("Bookmark current directory")},
                                              {bookmarks_edit, "bookmarks.edit", N_("Manage bookmarks")},
                                              {bookmarks_goto, "bookmarks.goto", N_("Go to bookmarked location")},
+                                             {command_execute, "command.execute", N_("Execute command")},
                                              {command_open_nautilus, "command.open_folder", N_("Open folder")},
                                              {command_open_terminal, "command.open_terminal", N_("Open terminal")},
                                              {command_root_mode, "command.root_mode", N_("Start GNOME Commander as root")},
@@ -164,7 +168,7 @@ static UserActionData user_actions_data[] = {
                                              {file_rename, "file.rename", N_("Rename files")},
                                              // {file_run, "file.run"},
                                              {file_sendto, "file.sendto", N_("Send files")},
-                                             {file_sync_dirs, "file.synchronize_directories", N_("Synchonize directories")},
+                                             {file_sync_dirs, "file.synchronize_directories", N_("Synchronize directories")},
                                              // {file_umount, "file.umount"},
                                              {file_view, "file.view", N_("View file")},
                                              {help_about, "help.about", N_("About GNOME Commander")},
@@ -185,6 +189,7 @@ static UserActionData user_actions_data[] = {
                                              {plugins_configure, "plugins.configure", N_("Configure plugins")},
                                              {plugins_execute_python, "plugins.execute_python", N_("Execute python plugin")},
                                              {view_back, "view.back", N_("Back one directory")},
+                                             {view_close_tab, "view.close_tab", N_("Close the current tab")},
                                              {view_equal_panes, "view.equal_panes", N_("Equal panel size")},
                                              {view_first, "view.first", N_("Back to the first directory")},
                                              {view_forward, "view.forward", N_("Forward one directory")},
@@ -194,8 +199,12 @@ static UserActionData user_actions_data[] = {
                                              {view_in_left_pane, "view.in_left_pane", N_("Open directory in the left window")},
                                              {view_in_right_pane, "view.in_right_pane", N_("Open directory in the right window")},
                                              {view_last, "view.last", N_("Forward to the last directory")},
+                                             {view_new_tab, "view.new_tab", N_("Open directory in a new tab")},
                                              {view_refresh, "view.refresh", N_("Refresh")},
                                              {view_root, "view.root", N_("Root directory")},
+#if 0
+                                             {view_terminal, "view.terminal", N_("Show terminal")},
+#endif
                                              {view_up, "view.up", N_("Up one directory")},
                                             };
 
@@ -240,7 +249,7 @@ void GnomeCmdUserActions::init()
     if (!registered("edit.search"))
     {
         register_action(GDK_MOD1_MASK, GDK_F7, "edit.search");
-#ifdef HAVE_GTK_2_10
+#if GTK_CHECK_VERSION (2, 10, 0)
         register_action(GDK_SUPER_MASK, GDK_F, "edit.search");
 #else
         register_action(GDK_MOD4_MASK, GDK_F, "edit.search");
@@ -248,10 +257,7 @@ void GnomeCmdUserActions::init()
     }
 
     if (!registered("file.advrename"))
-    {
         register_action(GDK_CONTROL_MASK, GDK_M, "file.advrename");
-        register_action(GDK_CONTROL_MASK, GDK_T, "file.advrename");
-    }
 
     if (!registered("file.create_symlink"))
         register_action(GDK_CONTROL_MASK | GDK_SHIFT_MASK, GDK_F5, "file.create_symlink");
@@ -268,6 +274,9 @@ void GnomeCmdUserActions::init()
     if (!registered("file.internal_view"))
         register_action(GDK_SHIFT_MASK, GDK_F3, "file.internal_view");
 
+    if (!registered("mark.compare_directories"))
+        register_action(GDK_SHIFT_MASK, GDK_F2, "mark.compare_directories");
+
     if (!registered("mark.select_all"))
     {
         register_action(GDK_CONTROL_MASK, GDK_A, "mark.select_all");
@@ -325,6 +334,15 @@ void GnomeCmdUserActions::init()
     if (!registered("view.refresh"))
         register_action(GDK_CONTROL_MASK, GDK_R, "view.refresh");
 
+    if (!registered("view.new_tab"))
+    {
+        unregister(GDK_CONTROL_MASK, GDK_T);                       // unregister CTRL+T as it was used previously for file.advrename
+        register_action(GDK_CONTROL_MASK, GDK_T, "view.new_tab");
+    }
+
+    if (!registered("view.close_tab"))
+        register_action(GDK_CONTROL_MASK, GDK_W, "view.close_tab");
+
     unregister(GDK_F9);                                 // unregister F9 if defined in [key-bindings]
     register_action(GDK_F9, "edit.search");             // and overwrite it with edit.search action
 
@@ -433,7 +451,7 @@ gboolean GnomeCmdUserActions::register_action(guint state, guint keyval, const g
     GdkEventKey event;
 
     event.keyval = keyval;
-#ifdef HAVE_GTK_2_10
+#if GTK_CHECK_VERSION (2, 10, 0)
     event.state = state & (GDK_SHIFT_MASK | GDK_CONTROL_MASK | GDK_MOD1_MASK | GDK_SUPER_MASK | GDK_HYPER_MASK | GDK_META_MASK);
 #else
     event.state = state & (GDK_SHIFT_MASK | GDK_CONTROL_MASK | GDK_MOD1_MASK | GDK_MOD4_MASK);
@@ -465,7 +483,7 @@ void GnomeCmdUserActions::unregister(guint state, guint keyval)
     GdkEventKey event;
 
     event.keyval = keyval;
-#ifdef HAVE_GTK_2_10
+#if GTK_CHECK_VERSION (2, 10, 0)
     event.state = state & (GDK_SHIFT_MASK | GDK_CONTROL_MASK | GDK_MOD1_MASK | GDK_SUPER_MASK | GDK_HYPER_MASK | GDK_META_MASK);
 #else
     event.state = state & (GDK_SHIFT_MASK | GDK_CONTROL_MASK | GDK_MOD1_MASK | GDK_MOD4_MASK);
@@ -497,7 +515,7 @@ gboolean GnomeCmdUserActions::registered(guint state, guint keyval)
     GdkEventKey event;
 
     event.keyval = keyval;
-#ifdef HAVE_GTK_2_10
+#if GTK_CHECK_VERSION (2, 10, 0)
     event.state = state & (GDK_SHIFT_MASK | GDK_CONTROL_MASK | GDK_MOD1_MASK | GDK_SUPER_MASK | GDK_HYPER_MASK | GDK_META_MASK);
 #else
     event.state = state & (GDK_SHIFT_MASK | GDK_CONTROL_MASK | GDK_MOD1_MASK | GDK_MOD4_MASK);
@@ -576,6 +594,30 @@ GtkTreeModel *gnome_cmd_user_actions_create_model ()
 }
 
 
+template <typename F>
+inline void get_file_list (string &s, GList *sfl, F f)
+{
+    vector<string> a;
+
+    for (GList *i = sfl; i; i = i->next)
+        a.push_back ((*f) (GNOME_CMD_FILE (i->data)));
+
+    join (s, a.begin(), a.end());
+}
+
+
+template <typename F, typename T>
+inline void get_file_list (string &s, GList *sfl, F f, T t)
+{
+    vector<string> a;
+
+    for (GList *i = sfl; i; i = i->next)
+        a.push_back ((*f) (GNOME_CMD_FILE (i->data), t));
+
+    join (s, a.begin(), a.end());
+}
+
+
 /***************************************/
 void no_action (GtkMenuItem *menuitem, gpointer not_used)
 {
@@ -660,14 +702,21 @@ void file_chown (GtkMenuItem *menuitem, gpointer not_used)
 
 void file_mkdir (GtkMenuItem *menuitem, gpointer not_used)
 {
-    gnome_cmd_file_selector_show_mkdir_dialog (get_fs (ACTIVE));
+    GnomeCmdDir *dir = get_fs (ACTIVE)->get_directory();
+    g_return_if_fail (GNOME_CMD_IS_DIR (dir));
+
+    GtkWidget *dialog = gnome_cmd_mkdir_dialog_new (dir);
+    g_return_if_fail (GNOME_CMD_IS_DIALOG (dialog));
+
+    gtk_widget_ref (dialog);
+    gtk_widget_show (dialog);
 }
 
 
 void file_create_symlink (GtkMenuItem *menuitem, gpointer not_used)
 {
     GnomeCmdFileSelector *inactive_fs = get_fs (INACTIVE);
-    GList *f = gnome_cmd_file_list_get_selected_files (get_fl (ACTIVE));
+    GList *f = get_fl (ACTIVE)->get_selected_files ();
     guint selected_files = g_list_length (f);
 
     if (selected_files > 1)
@@ -675,7 +724,7 @@ void file_create_symlink (GtkMenuItem *menuitem, gpointer not_used)
         gchar *msg = g_strdup_printf (ngettext("Create symbolic links of %i file in %s?",
                                                "Create symbolic links of %i files in %s?",
                                                selected_files),
-                                      selected_files, gnome_cmd_dir_get_display_path (gnome_cmd_file_selector_get_directory(inactive_fs)));
+                                      selected_files, gnome_cmd_dir_get_display_path (inactive_fs->get_directory()));
 
         gint choice = run_simple_dialog (GTK_WIDGET (main_win), TRUE, GTK_MESSAGE_QUESTION, msg, _("Create Symbolic Link"), 1, _("Cancel"), _("Create"), NULL);
 
@@ -686,8 +735,8 @@ void file_create_symlink (GtkMenuItem *menuitem, gpointer not_used)
     }
    else
    {
-        GnomeCmdFile *finfo = gnome_cmd_file_list_get_focused_file (get_fl (ACTIVE));
-        gnome_cmd_file_selector_create_symlink (inactive_fs, finfo);
+        GnomeCmdFile *f = get_fl (ACTIVE)->get_focused_file();
+        gnome_cmd_file_selector_create_symlink (inactive_fs, f);
    }
 }
 
@@ -700,35 +749,34 @@ void file_rename (GtkMenuItem *menuitem, gpointer not_used)
 
 void file_advrename (GtkMenuItem *menuitem, gpointer not_used)
 {
-    gnome_cmd_file_list_show_advrename_dialog (get_fl (ACTIVE));
+    GList *files = get_fl (ACTIVE)->get_selected_files();
+
+    if (files)
+    {
+        files = get_fl (ACTIVE)->sort_selection(files);
+
+        if (!main_win->advrename_dlg)
+        {
+            main_win->advrename_dlg = new GnomeCmdAdvrenameDialog(gnome_cmd_data.advrename_defaults);
+            // gtk_widget_ref (*main_win->advrename_dlg);      //  FIXME:  ???
+            main_win->advrename_dlg->set(files);
+            gtk_widget_show_all (*main_win->advrename_dlg);
+        }
+        else
+        {
+            main_win->advrename_dlg->set(files);
+            gtk_widget_show (*main_win->advrename_dlg);
+            // gdk_window_raise (GTK_WIDGET (main_win->advrename_dlg)->window);     //  FIXME:  bring dlg to top ???
+        }
+
+        g_list_free (files);
+    }
 }
 
 
 void file_sendto (GtkMenuItem *menuitem, gpointer not_used)
 {
-    string cmd = "nautilus-sendto";
-
-    GnomeCmdFileList *fl = get_fl (ACTIVE);
-    GList *sfl = gnome_cmd_file_list_get_selected_files (fl);
-    sfl = gnome_cmd_file_list_sort_selection (sfl, fl);
-
-    for (GList *i = sfl; i; i = i->next)
-    {
-        GnomeCmdFile *finfo = GNOME_CMD_FILE (i->data);
-
-        if (!finfo)
-            continue;
-
-        cmd += ' ';
-        cmd += (char *) gnome_cmd_file_get_quoted_real_path (finfo);
-    }
-
-    g_list_free (sfl);
-
-    g_print (_("running `%s'\n"), cmd.c_str());
-
-    if (cmd != "nautilus-sendto")
-        run_command (cmd.c_str(), FALSE);
+    command_execute (menuitem, (gpointer) "nautilus-sendto %s");
 }
 
 
@@ -740,7 +788,7 @@ void file_properties (GtkMenuItem *menuitem, gpointer not_used)
 
 void file_diff (GtkMenuItem *menuitem, gpointer not_used)
 {
-    if (!gnome_cmd_file_selector_is_local (get_fs (ACTIVE)))
+    if (!get_fs (ACTIVE)->is_local())
     {
         create_error_dialog (_("Operation not supported on remote file systems"));
         return;
@@ -748,7 +796,7 @@ void file_diff (GtkMenuItem *menuitem, gpointer not_used)
 
     GnomeCmdFileList *active_fl = get_fl (ACTIVE);
 
-    GList *sel_files = gnome_cmd_file_list_get_selected_files (active_fl);
+    GList *sel_files = active_fl->get_selected_files();
 
     string s;
 
@@ -758,7 +806,7 @@ void file_diff (GtkMenuItem *menuitem, gpointer not_used)
             return;
 
         case 1:
-            if (!gnome_cmd_file_selector_is_local (get_fs (INACTIVE)))
+            if (!get_fs (INACTIVE)->is_local())
                 create_error_dialog (_("Operation not supported on remote file systems"));
             else
                 if (!append_real_path (s, get_selected_file (ACTIVE)) || !append_real_path (s, get_selected_file (INACTIVE)))
@@ -767,7 +815,7 @@ void file_diff (GtkMenuItem *menuitem, gpointer not_used)
 
         case 2:
         case 3:
-            sel_files = gnome_cmd_file_list_sort_selection (sel_files, active_fl);
+            sel_files = active_fl->sort_selection(sel_files);
 
             for (GList *i = sel_files; i; i = i->next)
                 append_real_path (s, GNOME_CMD_FILE (i->data));
@@ -782,7 +830,7 @@ void file_diff (GtkMenuItem *menuitem, gpointer not_used)
 
     if (!s.empty())
     {
-        gchar *cmd = g_strdup_printf (gnome_cmd_data_get_differ (), s.c_str(), "");
+        gchar *cmd = g_strdup_printf (gnome_cmd_data.get_differ(), s.c_str(), "");
 
         g_print (_("running `%s'\n"), cmd);
         run_command (cmd, FALSE);
@@ -797,14 +845,14 @@ void file_sync_dirs (GtkMenuItem *menuitem, gpointer not_used)
     GnomeCmdFileSelector *active_fs = get_fs (ACTIVE);
     GnomeCmdFileSelector *inactive_fs = get_fs (INACTIVE);
 
-    if (!gnome_cmd_file_selector_is_local (active_fs) || !gnome_cmd_file_selector_is_local (inactive_fs))
+    if (!active_fs->is_local() || !inactive_fs->is_local())
     {
         create_error_dialog (_("Operation not supported on remote file systems"));
         return;
     }
 
-    GnomeVFSURI *active_dir_uri = gnome_cmd_dir_get_uri (gnome_cmd_file_selector_get_directory (active_fs));
-    GnomeVFSURI *inactive_dir_uri = gnome_cmd_dir_get_uri (gnome_cmd_file_selector_get_directory (inactive_fs));
+    GnomeVFSURI *active_dir_uri = gnome_cmd_dir_get_uri (active_fs->get_directory());
+    GnomeVFSURI *inactive_dir_uri = gnome_cmd_dir_get_uri (inactive_fs->get_directory());
     gchar *active_dir = gnome_vfs_unescape_string (gnome_vfs_uri_get_path (active_dir_uri), NULL);
     gchar *inactive_dir = gnome_vfs_unescape_string (gnome_vfs_uri_get_path (inactive_dir_uri), NULL);
 
@@ -819,7 +867,7 @@ void file_sync_dirs (GtkMenuItem *menuitem, gpointer not_used)
     g_free (active_dir);
     g_free (inactive_dir);
 
-    gchar *cmd = g_strdup_printf (gnome_cmd_data_get_differ (), s.c_str(), "");
+    gchar *cmd = g_strdup_printf (gnome_cmd_data.get_differ(), s.c_str(), "");
 
     g_print (_("running `%s'\n"), cmd);
     run_command (cmd, FALSE);
@@ -832,7 +880,7 @@ void file_exit (GtkMenuItem *menuitem, gpointer not_used)
 {
     gint x, y;
 
-    switch (gnome_cmd_data_get_main_win_state())
+    switch (gnome_cmd_data.main_win_state)
     {
         case GDK_WINDOW_STATE_MAXIMIZED:
         case GDK_WINDOW_STATE_FULLSCREEN:
@@ -871,7 +919,7 @@ void edit_cap_paste (GtkMenuItem *menuitem, gpointer not_used)
 void edit_search (GtkMenuItem *menuitem, gpointer not_used)
 {
     GnomeCmdFileSelector *fs = get_fs (ACTIVE);
-    GtkWidget *dialog = gnome_cmd_search_dialog_new (gnome_cmd_file_selector_get_directory (fs));
+    GtkWidget *dialog = gnome_cmd_search_dialog_new (fs->get_directory());
     gtk_widget_ref (dialog);
     gtk_widget_show (dialog);
 }
@@ -885,50 +933,159 @@ void edit_quick_search (GtkMenuItem *menuitem, gpointer not_used)
 
 void edit_filter (GtkMenuItem *menuitem, gpointer not_used)
 {
-    gnome_cmd_file_selector_show_filter (get_fs (ACTIVE), 0);
+    get_fs (ACTIVE)->show_filter();
 }
 
 
 void edit_copy_fnames (GtkMenuItem *menuitem, gpointer not_used)
 {
-    static gchar sep[] = " ";
-
     GdkModifierType mask;
 
     gdk_window_get_pointer (NULL, NULL, NULL, &mask);
 
     GnomeCmdFileList *fl = get_fl (ACTIVE);
-    GList *sfl = gnome_cmd_file_list_get_selected_files (fl);
-    gchar **fnames = g_new (char *, g_list_length (sfl) + 1);
-    gchar **f = fnames;
+    GList *sfl = fl->get_selected_files();
+    sfl = fl->sort_selection(sfl);
 
-    sfl = gnome_cmd_file_list_sort_selection (sfl, fl);
+    string fnames;
 
-    for (GList *i = sfl; i; i = i->next)
-    {
-        GnomeCmdFile *finfo = GNOME_CMD_FILE (i->data);
+    fnames.reserve(2000);
 
-        if (finfo)
-          *f++ = (mask & GDK_SHIFT_MASK) ? (char *) gnome_cmd_file_get_real_path (finfo) :
-                                           (char *) gnome_cmd_file_get_name (finfo);
-    }
+    if (state_is_blank (mask))
+        get_file_list (fnames, sfl, gnome_cmd_file_get_name);
+    else
+        if (state_is_shift (mask))
+            get_file_list (fnames, sfl, gnome_cmd_file_get_real_path);
+        else
+            if (state_is_alt (mask))
+                get_file_list (fnames, sfl, gnome_cmd_file_get_uri_str, GNOME_VFS_URI_HIDE_PASSWORD);
 
-    *f = NULL;
+    gtk_clipboard_set_text (gtk_clipboard_get (GDK_SELECTION_CLIPBOARD), fnames.c_str(), -1);
 
-    gchar *s = g_strjoinv(sep,fnames);
-
-    gtk_clipboard_set_text (gtk_clipboard_get (GDK_SELECTION_CLIPBOARD), s, -1);
-
-    g_free (s);
     g_list_free (sfl);
-    g_free (fnames);
 }
 
 
 /************** Command Menu **************/
+void command_execute (GtkMenuItem *menuitem, gpointer command)
+{
+    g_return_if_fail (command != NULL);
+
+    DEBUG ('g', "invoking: %s\n", command);
+
+    string cmd;
+    string filename;
+    string quoted_filename;
+    string file_path;
+    string quoted_file_path;
+    string dir_path;
+    string quoted_dir_path;
+    string uri;
+
+    GnomeCmdDir *dir = NULL;
+
+    GnomeCmdFileList *fl = get_fl (ACTIVE);
+    GList *sfl = fl->get_selected_files();
+    GList *i = sfl = fl->sort_selection(sfl);
+
+    if (i)
+    {
+        dir = gnome_cmd_file_get_parent_dir (GNOME_CMD_FILE (i->data));
+        i = i->next;
+    }
+
+    for (; i && gnome_cmd_file_get_parent_dir (GNOME_CMD_FILE (i->data))==dir; i=i->next);
+
+    if (i)
+        dir = NULL;
+
+    if (dir)
+    {
+        stringify (dir_path, gnome_cmd_file_get_real_path (GNOME_CMD_FILE (dir)));
+        stringify (quoted_dir_path, gnome_cmd_file_get_quoted_real_path (GNOME_CMD_FILE (dir)));
+    }
+
+    gboolean percent = FALSE;
+
+    cmd.reserve(2000);
+
+    for (const char *s=(const char *) command; *s; ++s)
+    {
+        if (!percent)
+        {
+            percent = *s=='%';
+
+            if (!percent)
+                cmd += *s;
+
+            continue;
+        }
+
+        switch (*s)
+        {
+            case 'f':           // %f  file name (or list for multiple selections)
+                if (filename.empty())
+                    get_file_list (filename, sfl, gnome_cmd_file_get_name);
+                cmd += filename;
+                break;
+
+            case 'F':           // %F  quoted filename (or list for multiple selections)
+                if (quoted_filename.empty())
+                    get_file_list (quoted_filename, sfl, gnome_cmd_file_get_quoted_name);
+                cmd += quoted_filename;
+                break;
+
+            case 'p':           // %p  full file system path (or list for multiple selections)
+                if (file_path.empty())
+                    get_file_list (file_path, sfl, gnome_cmd_file_get_real_path);
+                cmd += file_path;
+                break;
+
+            case 'P':           // %P  quoted full file system path (or list for multiple selections)
+            case 's':           // %s  synonym for %P (for compatibility with previous versions of gcmd)
+                if (quoted_file_path.empty())
+                    get_file_list (quoted_file_path, sfl, gnome_cmd_file_get_quoted_real_path);
+                cmd += quoted_file_path;
+                break;
+
+            case 'u':           // %u  fully qualified URI for the file (or list for multiple selections)
+                if (uri.empty())
+                    get_file_list (uri, sfl, gnome_cmd_file_get_uri_str, GNOME_VFS_URI_HIDE_NONE);
+                cmd += uri;
+                break;
+
+            case 'd':           // %d  full path to the directory containing file
+                cmd += dir_path;
+                break;
+
+            case 'D':           // %D  quoted full path to the directory containg file
+                cmd += quoted_dir_path;
+                break;
+
+            default:
+                cmd += '%';
+            case '%':           // %%  percent sign
+                cmd += *s;
+                break;
+        }
+
+        percent = FALSE;
+    }
+
+    if (percent)
+        cmd += '%';
+
+    DEBUG ('g', "running: %s\n", cmd.c_str());
+
+    gnome_execute_shell (dir && gnome_cmd_dir_is_local (dir) ? dir_path.c_str() : NULL, cmd.c_str());
+
+    g_list_free (sfl);
+}
+
+
 void command_open_terminal (GtkMenuItem *menuitem, gpointer not_used)
 {
-    gchar *dpath = gnome_cmd_file_get_real_path (GNOME_CMD_FILE (gnome_cmd_file_selector_get_directory (get_fs (ACTIVE))));
+    gchar *dpath = gnome_cmd_file_get_real_path (GNOME_CMD_FILE (get_fs (ACTIVE)->get_directory()));
 
     gnome_execute_terminal_shell (dpath, NULL);
     g_free (dpath);
@@ -959,15 +1116,15 @@ inline void open_uri_in_nautilus (gchar *uri)
 
 void command_open_nautilus (GtkMenuItem *menuitem, gpointer not_used)
 {
-    GnomeCmdFile *f = gnome_cmd_file_list_get_selected_file (get_fl (ACTIVE));
+    GnomeCmdFile *f = get_fl (ACTIVE)->get_selected_file();
 
-    open_uri_in_nautilus (gnome_cmd_file_get_uri_str (GNOME_CMD_IS_DIR (f) ? f : GNOME_CMD_FILE (gnome_cmd_file_selector_get_directory (get_fs (ACTIVE)))));
+    open_uri_in_nautilus (gnome_cmd_file_get_uri_str (GNOME_CMD_IS_DIR (f) ? f : GNOME_CMD_FILE (get_fs (ACTIVE)->get_directory())));
 }
 
 
 void command_open_nautilus_in_cwd (GtkMenuItem *menuitem, gpointer not_used)
 {
-    open_uri_in_nautilus (gnome_cmd_file_get_uri_str (GNOME_CMD_FILE (gnome_cmd_file_selector_get_directory (get_fs (ACTIVE)))));
+    open_uri_in_nautilus (gnome_cmd_file_get_uri_str (GNOME_CMD_FILE (get_fs (ACTIVE)->get_directory())));
 }
 
 
@@ -975,9 +1132,7 @@ void command_root_mode (GtkMenuItem *menuitem, gpointer not_used)
 {
     char *su;
 
-    su = g_find_program_in_path ("gksudo");
-    if  (!su)
-        su = g_find_program_in_path ("gksu");
+    su = g_find_program_in_path ("gksu");
     if  (!su)
         su = g_find_program_in_path ("kdesu");
 
@@ -1005,25 +1160,25 @@ void command_root_mode (GtkMenuItem *menuitem, gpointer not_used)
 /************** Mark Menu **************/
 void mark_toggle (GtkMenuItem *menuitem, gpointer not_used)
 {
-    gnome_cmd_file_list_toggle (get_fl (ACTIVE));
+    get_fl (ACTIVE)->toggle();
 }
 
 
 void mark_toggle_and_step (GtkMenuItem *menuitem, gpointer not_used)
 {
-    gnome_cmd_file_list_toggle_and_step (get_fl (ACTIVE));
+    get_fl (ACTIVE)->toggle_and_step();
 }
 
 
 void mark_select_all (GtkMenuItem *menuitem, gpointer not_used)
 {
-    gnome_cmd_file_list_select_all (get_fl (ACTIVE));
+    get_fl (ACTIVE)->select_all();
 }
 
 
 void mark_unselect_all (GtkMenuItem *menuitem, gpointer not_used)
 {
-    gnome_cmd_file_list_unselect_all (get_fl (ACTIVE));
+    get_fl (ACTIVE)->unselect_all();
 }
 
 
@@ -1041,31 +1196,31 @@ void mark_unselect_with_pattern (GtkMenuItem *menuitem, gpointer not_used)
 
 void mark_invert_selection (GtkMenuItem *menuitem, gpointer not_used)
 {
-    gnome_cmd_file_list_invert_selection (get_fl (ACTIVE));
+    get_fl (ACTIVE)->invert_selection();
 }
 
 
 void mark_select_all_with_same_extension (GtkMenuItem *menuitem, gpointer not_used)
 {
-    gnome_cmd_file_list_select_all_with_same_extension (get_fl (ACTIVE));
+    get_fl (ACTIVE)->select_all_with_same_extension();
 }
 
 
 void mark_unselect_all_with_same_extension (GtkMenuItem *menuitem, gpointer not_used)
 {
-    gnome_cmd_file_list_unselect_all_with_same_extension (get_fl (ACTIVE));
+    get_fl (ACTIVE)->unselect_all_with_same_extension();
 }
 
 
 void mark_restore_selection (GtkMenuItem *menuitem, gpointer not_used)
 {
-    gnome_cmd_file_list_restore_selection (get_fl (ACTIVE));
+    get_fl (ACTIVE)->restore_selection();
 }
 
 
 void mark_compare_directories (GtkMenuItem *menuitem, gpointer not_used)
 {
-    gnome_cmd_file_list_compare_directories ();
+    gnome_cmd_file_list_compare_directories (get_fl (ACTIVE), get_fl (INACTIVE));
 }
 
 
@@ -1076,9 +1231,20 @@ void view_conbuttons (GtkMenuItem *menuitem, gpointer not_used)
     if (!GTK_WIDGET_REALIZED (main_win)) return;
 
     GtkCheckMenuItem *checkitem = (GtkCheckMenuItem *) menuitem;
-    gnome_cmd_data_set_conbuttons_visibility (checkitem->active);
-    gnome_cmd_file_selector_update_conbuttons_visibility (get_fs (ACTIVE));
-    gnome_cmd_file_selector_update_conbuttons_visibility (get_fs (INACTIVE));
+    gnome_cmd_data.conbuttons_visibility = checkitem->active;
+    get_fs (ACTIVE)->update_conbuttons_visibility();
+    get_fs (INACTIVE)->update_conbuttons_visibility();
+}
+
+
+void view_concombo (GtkMenuItem *menuitem, gpointer not_used)
+{
+    if (!GTK_WIDGET_REALIZED (main_win)) return;
+
+    GtkCheckMenuItem *checkitem = (GtkCheckMenuItem *) menuitem;
+    gnome_cmd_data.concombo_visibility = checkitem->active;
+    get_fs (ACTIVE)->update_concombo_visibility();
+    get_fs (INACTIVE)->update_concombo_visibility();
 }
 
 
@@ -1087,7 +1253,7 @@ void view_toolbar (GtkMenuItem *menuitem, gpointer not_used)
     if (!GTK_WIDGET_REALIZED (main_win)) return;
 
     GtkCheckMenuItem *checkitem = (GtkCheckMenuItem *) menuitem;
-    gnome_cmd_data_set_toolbar_visibility (checkitem->active);
+    gnome_cmd_data.toolbar_visibility = checkitem->active;
     gnome_cmd_main_win_update_toolbar_visibility (main_win);
 }
 
@@ -1097,7 +1263,7 @@ void view_buttonbar (GtkMenuItem *menuitem, gpointer not_used)
     if (!GTK_WIDGET_REALIZED (main_win)) return;
 
     GtkCheckMenuItem *checkitem = (GtkCheckMenuItem *) menuitem;
-    gnome_cmd_data_set_buttonbar_visibility (checkitem->active);
+    gnome_cmd_data.buttonbar_visibility = checkitem->active;
     gnome_cmd_main_win_update_buttonbar_visibility (main_win);
 }
 
@@ -1107,7 +1273,7 @@ void view_cmdline (GtkMenuItem *menuitem, gpointer not_used)
     if (!GTK_WIDGET_REALIZED (main_win)) return;
 
     GtkCheckMenuItem *checkitem = (GtkCheckMenuItem *) menuitem;
-    gnome_cmd_data_set_cmdline_visibility (checkitem->active);
+    gnome_cmd_data.cmdline_visibility = checkitem->active;
     gnome_cmd_main_win_update_cmdline_visibility (main_win);
 }
 
@@ -1117,9 +1283,9 @@ void view_hidden_files (GtkMenuItem *menuitem, gpointer not_used)
     if (!GTK_WIDGET_REALIZED (main_win)) return;
 
     GtkCheckMenuItem *checkitem = (GtkCheckMenuItem *) menuitem;
-    gnome_cmd_data_get_filter_settings()->hidden = !checkitem->active;
-    gnome_cmd_file_selector_reload (get_fs (ACTIVE));
-    gnome_cmd_file_selector_reload (get_fs (INACTIVE));
+    gnome_cmd_data.filter_settings.hidden = !checkitem->active;
+    get_fs (ACTIVE)->reload();
+    get_fs (INACTIVE)->reload();
 }
 
 
@@ -1128,45 +1294,45 @@ void view_backup_files (GtkMenuItem *menuitem, gpointer not_used)
     if (!GTK_WIDGET_REALIZED (main_win)) return;
 
     GtkCheckMenuItem *checkitem = (GtkCheckMenuItem *) menuitem;
-    gnome_cmd_data_get_filter_settings()->backup = !checkitem->active;
-    gnome_cmd_file_selector_reload (get_fs (ACTIVE));
-    gnome_cmd_file_selector_reload (get_fs (INACTIVE));
+    gnome_cmd_data.filter_settings.backup = !checkitem->active;
+    get_fs (ACTIVE)->reload();
+    get_fs (INACTIVE)->reload();
 }
 
 
 void view_up (GtkMenuItem *menuitem, gpointer not_used)
 {
-    gnome_cmd_file_selector_goto_directory (get_fs (ACTIVE), "..");
+    get_fs (ACTIVE)->goto_directory("..");
 }
 
 
 void view_first (GtkMenuItem *menuitem, gpointer not_used)
 {
-    gnome_cmd_file_selector_first (get_fs (ACTIVE));
+    get_fs (ACTIVE)->first();
 }
 
 
 void view_back (GtkMenuItem *menuitem, gpointer not_used)
 {
-    gnome_cmd_file_selector_back (get_fs (ACTIVE));
+    get_fs (ACTIVE)->back();
 }
 
 
 void view_forward (GtkMenuItem *menuitem, gpointer not_used)
 {
-    gnome_cmd_file_selector_forward (get_fs (ACTIVE));
+    get_fs (ACTIVE)->forward();
 }
 
 
 void view_last (GtkMenuItem *menuitem, gpointer not_used)
 {
-    gnome_cmd_file_selector_last (get_fs (ACTIVE));
+    get_fs (ACTIVE)->last();
 }
 
 
 void view_refresh (GtkMenuItem *menuitem, gpointer not_used)
 {
-    gnome_cmd_file_selector_reload (get_fs (ACTIVE));
+    get_fs (ACTIVE)->reload();
 }
 
 
@@ -1178,38 +1344,48 @@ void view_equal_panes (GtkMenuItem *menuitem, gpointer not_used)
 
 void view_in_left_pane (GtkMenuItem *menuitem, gpointer not_used)
 {
-    gnome_cmd_file_selector_set_same_directory ((GnomeCmdMainWin *) GTK_WIDGET (main_win), LEFT, RIGHT);
+    gnome_cmd_file_selector_set_directory_to_opposite ((GnomeCmdMainWin *) GTK_WIDGET (main_win), LEFT);
 }
 
 
 void view_in_right_pane (GtkMenuItem *menuitem, gpointer not_used)
 {
-    gnome_cmd_file_selector_set_same_directory ((GnomeCmdMainWin *) GTK_WIDGET (main_win), RIGHT, LEFT);
+    gnome_cmd_file_selector_set_directory_to_opposite ((GnomeCmdMainWin *) GTK_WIDGET (main_win), RIGHT);
 }
 
 
 void view_in_active_pane (GtkMenuItem *menuitem, gpointer not_used)
 {
-    gnome_cmd_file_selector_set_same_directory ((GnomeCmdMainWin *) GTK_WIDGET (main_win), ACTIVE, INACTIVE);
+    gnome_cmd_file_selector_set_directory_to_opposite ((GnomeCmdMainWin *) GTK_WIDGET (main_win), ACTIVE);
 }
 
 
 void view_in_inactive_pane (GtkMenuItem *menuitem, gpointer not_used)
 {
-    gnome_cmd_file_selector_set_same_directory ((GnomeCmdMainWin *) GTK_WIDGET (main_win), INACTIVE, ACTIVE);
+    gnome_cmd_file_selector_set_directory_to_opposite ((GnomeCmdMainWin *) GTK_WIDGET (main_win), INACTIVE);
 }
 
 
 void view_home (GtkMenuItem *menuitem, gpointer not_used)
 {
-    gnome_cmd_file_selector_set_connection (get_fs (ACTIVE), get_home_con ());
-    gnome_cmd_file_selector_goto_directory (get_fs (ACTIVE), "~");
+    get_fs (ACTIVE)->set_connection(get_home_con ());
+    get_fs (ACTIVE)->goto_directory("~");
 }
 
 
 void view_root (GtkMenuItem *menuitem, gpointer not_used)
 {
-    gnome_cmd_file_selector_goto_directory (get_fs (ACTIVE), "/");
+    get_fs (ACTIVE)->goto_directory("/");
+}
+
+
+void view_new_tab (GtkMenuItem *menuitem, gpointer not_used)
+{
+}
+
+
+void view_close_tab (GtkMenuItem *menuitem, gpointer not_used)
+{
 }
 
 
@@ -1252,7 +1428,7 @@ void connections_new (GtkMenuItem *menuitem, gpointer not_used)
 
 void connections_change (GtkMenuItem *menuitem, gpointer con)           // this function is NOT exposed to user as UserAction
 {
-    gnome_cmd_file_selector_set_connection (get_fs (ACTIVE), (GnomeCmdCon *) con);
+    get_fs (ACTIVE)->set_connection((GnomeCmdCon *) con);
 }
 
 
@@ -1261,14 +1437,12 @@ void connections_close (GtkMenuItem *menuitem, gpointer con)            // this
     GnomeCmdFileSelector *active = get_fs (ACTIVE);
     GnomeCmdFileSelector *inactive = get_fs (INACTIVE);
 
-    GnomeCmdCon *c1 = gnome_cmd_file_selector_get_connection (active);
-    GnomeCmdCon *c2 = gnome_cmd_file_selector_get_connection (inactive);
     GnomeCmdCon *home = get_home_con ();
 
-    if (con == c1)
-        gnome_cmd_file_selector_set_connection (active, home);
-    if (con == c2)
-        gnome_cmd_file_selector_set_connection (inactive, home);
+    if (con == active->get_connection())
+        active->set_connection(home);
+    if (con == inactive->get_connection())
+        inactive->set_connection(home);
 
     gnome_cmd_con_close ((GnomeCmdCon *) con);
 }
@@ -1276,7 +1450,7 @@ void connections_close (GtkMenuItem *menuitem, gpointer con)            // this
 
 void connections_close_current (GtkMenuItem *menuitem, gpointer not_used)
 {
-    GnomeCmdCon *con = gnome_cmd_file_selector_get_connection (get_fs (ACTIVE));
+    GnomeCmdCon *con = get_fs (ACTIVE)->get_connection();
 
     connections_close (menuitem, con);
 }
@@ -1386,7 +1560,7 @@ void help_help (GtkMenuItem *menuitem, gpointer not_used)
 
 void help_keyboard (GtkMenuItem *menuitem, gpointer not_used)
 {
-    gnome_cmd_help_display ("gnome-commander.xml","gnome-commander-keyboard");
+    gnome_cmd_help_display ("gnome-commander.xml", "gnome-commander-keyboard");
 }
 
 
@@ -1394,8 +1568,13 @@ void help_web (GtkMenuItem *menuitem, gpointer not_used)
 {
     GError *error = NULL;
 
+#if GTK_CHECK_VERSION (2, 14, 0)
+    if (!gtk_show_uri (NULL, "http://www.nongnu.org/gcmd/", GDK_CURRENT_TIME, &error))
+        gnome_cmd_error_message (_("There was an error opening home page."), error);
+#else
     if (!gnome_url_show ("http://www.nongnu.org/gcmd/", &error))
         gnome_cmd_error_message (_("There was an error opening home page."), error);
+#endif
 }
 
 
@@ -1403,8 +1582,13 @@ void help_problem (GtkMenuItem *menuitem, gpointer not_used)
 {
     GError *error = NULL;
 
+#if GTK_CHECK_VERSION (2, 14, 0)
+    if (!gtk_show_uri (NULL, "http://bugzilla.gnome.org/browse.cgi?product=gnome-commander", GDK_CURRENT_TIME, &error))
+        gnome_cmd_error_message (_("There was an error reporting problem."), error);
+#else
     if (!gnome_url_show("http://bugzilla.gnome.org/browse.cgi?product=gnome-commander", &error))
         gnome_cmd_error_message (_("There was an error reporting problem."), error);
+#endif
 }
 
 
@@ -1423,7 +1607,8 @@ void help_about (GtkMenuItem *menuitem, gpointer not_used)
         NULL
     };
 
-    static const gchar copyright[] = "Copyright \xc2\xa9 2001-2006 Marcus Bjurman";
+    static const gchar copyright[] = "Copyright \xc2\xa9 2001-2006 Marcus Bjurman\n"
+                                     "Copyright \xc2\xa9 2007-2009 Piotr Eljasiak";
 
     static const gchar comments[] = N_("A fast and powerful file manager for the GNOME desktop");
 
