@@ -5,6 +5,7 @@
     Part of
         GNOME Commander - A GNOME based file manager
         Copyright (C) 2001-2006 Marcus Bjurman
+        Copyright (C) 2007-2009 Piotr Eljasiak
 
     Partly based on "gtkhex.c" module from "GHex2" Project.
     Author: Jaka Mocnik <jaka@gnu.org>
@@ -25,7 +26,6 @@
 */
 
 #include <config.h>
-
 #include <errno.h>
 #include <unistd.h>
 #include <string.h>
@@ -42,6 +42,9 @@
 #include "datapresentation.h"
 #include "text-render.h"
 
+using namespace std;
+
+
 #define HEXDUMP_FIXED_LIMIT 16
 #define MAX_CLIPBOARD_COPY_LENGTH 0xFFFFFF
 
@@ -63,7 +66,7 @@ typedef void (*copy_to_clipboard_proc)(TextRender *obj, offset_type start_offset
 
 
 // Class Private Data
-struct _TextRenderPrivate
+struct TextRenderPrivate
 {
     guint8 button; // The button pressed in "button_press_event"
 
@@ -167,7 +170,7 @@ static offset_type hex_mode_pixel_to_offset(TextRender *obj, int x, int y, gbool
     public functions
     (defined in the header file)
 *****************************************/
-GtkType text_render_get_type (void)
+GtkType text_render_get_type ()
 {
     static GtkType type = 0;
     if (type == 0)
@@ -188,7 +191,7 @@ GtkType text_render_get_type (void)
     return type;
 }
 
-GtkWidget* text_render_new (void)
+GtkWidget* text_render_new ()
 {
     TextRender *w = (TextRender *) gtk_type_new (text_render_get_type ());
 
@@ -204,7 +207,7 @@ void text_render_set_h_adjustment (TextRender *obj, GtkAdjustment *adjustment)
     if (obj->priv->h_adjustment)
     {
         gtk_signal_disconnect_by_data (GTK_OBJECT (obj->priv->h_adjustment), (gpointer) obj);
-        gtk_object_unref (GTK_OBJECT (obj->priv->h_adjustment));
+        g_object_unref (obj->priv->h_adjustment);
     }
 
     obj->priv->h_adjustment = adjustment;
@@ -233,7 +236,7 @@ void text_render_set_v_adjustment (TextRender *obj, GtkAdjustment *adjustment)
     if (obj->priv->v_adjustment)
     {
         gtk_signal_disconnect_by_data (GTK_OBJECT (obj->priv->v_adjustment), (gpointer) obj);
-        gtk_object_unref (GTK_OBJECT (obj->priv->v_adjustment));
+        g_object_unref (obj->priv->v_adjustment);
     }
 
     obj->priv->v_adjustment = adjustment;
@@ -256,8 +259,8 @@ void text_render_set_v_adjustment (TextRender *obj, GtkAdjustment *adjustment)
 
 static void text_render_class_init (TextRenderClass *klass)
 {
-    GtkObjectClass *object_class = (GtkObjectClass *) klass;
-    GtkWidgetClass *widget_class = (GtkWidgetClass *) klass;
+    GtkObjectClass *object_class = GTK_OBJECT_CLASS (klass);
+    GtkWidgetClass *widget_class = GTK_WIDGET_CLASS (klass);
 
     parent_class = (GtkWidgetClass *) gtk_type_class (gtk_widget_get_type ());
 
@@ -276,7 +279,7 @@ static void text_render_class_init (TextRenderClass *klass)
     widget_class->realize = text_render_realize;
 
     text_render_signals[TEXT_STATUS_CHANGED] =
-        gtk_signal_new ("text_status_changed",
+        gtk_signal_new ("text-status-changed",
             GTK_RUN_LAST,
             G_OBJECT_CLASS_TYPE (object_class),
             GTK_SIGNAL_OFFSET (TextRenderClass, text_status_changed),
@@ -321,7 +324,7 @@ static void text_render_init (TextRender *w)
 
     w->priv->fixed_font_name = g_strdup ("Monospace");
 
-    g_signal_connect(G_OBJECT (w), "key_press_event", G_CALLBACK (text_render_key_pressed), NULL);
+    g_signal_connect(G_OBJECT (w), "key-press-event", G_CALLBACK (text_render_key_pressed), NULL);
 
     w->priv->layout = gtk_widget_create_pango_layout(GTK_WIDGET (w), NULL);
 
@@ -363,11 +366,11 @@ static void text_render_destroy (GtkObject *object)
         w->priv->fixed_font_name = NULL;
 
         if (w->priv->v_adjustment)
-            gtk_object_unref (GTK_OBJECT(w->priv->v_adjustment));
+            g_object_unref (w->priv->v_adjustment);
         w->priv->v_adjustment = NULL;
 
         if (w->priv->h_adjustment)
-            gtk_object_unref (GTK_OBJECT(w->priv->h_adjustment));
+            g_object_unref (w->priv->h_adjustment);
         w->priv->h_adjustment = NULL;
 
         g_free(w->priv->encoding);
@@ -681,7 +684,7 @@ static gboolean text_render_button_press(GtkWidget *widget, GdkEventButton *even
     {
         gtk_grab_add (widget);
         w->priv->button = event->button;
-        w->priv->marker_start  = w->priv->pixel_to_offset(w, (int)event->x, (int)event->y, TRUE);
+        w->priv->marker_start  = w->priv->pixel_to_offset(w, (int) event->x, (int) event->y, TRUE);
     }
 
     return FALSE;
@@ -770,7 +773,7 @@ static void text_render_h_adjustment_update (TextRender *obj)
         gtk_signal_emit_by_name (GTK_OBJECT (obj->priv->h_adjustment), "value-changed");
     }
 
-    obj->priv->column = (int)new_value;
+    obj->priv->column = (int) new_value;
 
     text_render_redraw(obj);
 }
@@ -828,7 +831,7 @@ static void text_render_v_adjustment_update (TextRender *obj)
         return;
 
     if (obj->priv->dp)
-        new_value = gv_align_offset_to_line_start(obj->priv->dp, (offset_type)new_value);
+        new_value = gv_align_offset_to_line_start(obj->priv->dp, (offset_type) new_value);
 
     if (new_value != obj->priv->v_adjustment->value)
     {
@@ -921,7 +924,7 @@ static void text_render_internal_load(TextRender *w)
     gv_set_fixed_count(w->priv->dp, w->priv->fixed_limit);
     gv_set_tab_size(w->priv->dp, w->priv->tab_size);
 
-    text_render_set_display_mode(w, TR_DISP_MODE_TEXT);
+    text_render_set_display_mode (w, TR_DISP_MODE_TEXT);
 
     text_render_update_adjustments_limits(w);
 }
@@ -1095,7 +1098,7 @@ static guint get_max_char_width(GtkWidget *widget, PangoFontDescription *font_de
         // Check if the char is displayable. Caused trouble to pango
         if (is_displayable((guchar)i))
         {
-            sprintf (str, "%c", (gchar)i);
+            sprintf (str, "%c", (gchar) i);
             pango_layout_set_text(layout, str, -1);
             pango_layout_get_pixel_extents (layout, NULL, &logical_rect);
         }
@@ -1244,7 +1247,7 @@ static int text_render_utf8_print_char(TextRender *w, char_type value)
 }
 
 
-void  text_render_set_display_mode(TextRender *w, TEXTDISPLAYMODE mode)
+void  text_render_set_display_mode (TextRender *w, TEXTDISPLAYMODE mode)
 {
     g_return_if_fail (w!=NULL);
     g_return_if_fail (IS_TEXT_RENDER(w));
@@ -1920,10 +1923,10 @@ static void hex_mode_copy_to_clipboard(TextRender *obj, offset_type start_offset
         char_type value = gv_input_mode_get_raw_byte(obj->priv->im, current);
         if (value==INVALID_CHAR)
             break;
-        text_render_utf8_printf(obj, "%02x ", (unsigned char)value);
+        text_render_utf8_printf(obj, "%02x ", (unsigned char) value);
     }
 
-    gtk_clipboard_set_text(clip, (const gchar*) obj->priv->utf8buf, obj->priv->utf8buf_length);
+    gtk_clipboard_set_text(clip, (const gchar *) obj->priv->utf8buf, obj->priv->utf8buf_length);
 }
 
 
