@@ -5,6 +5,7 @@
     Part of
         GNOME Commander - A GNOME based file manager
         Copyright (C) 2001-2006 Marcus Bjurman
+        Copyright (C) 2007-2009 Piotr Eljasiak
 
     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -22,7 +23,6 @@
 */
 
 #include <config.h>
-
 #include <glib.h>
 #include <libgnomeui/libgnomeui.h>
 #include <libgnome/libgnome.h>
@@ -30,6 +30,8 @@
 #include "gvtypes.h"
 #include "viewer-utils.h"
 
+using namespace std;
+
 
 gchar *gviewer_get_string (const gchar *path, const gchar *def)
 {
@@ -61,60 +63,6 @@ gboolean gviewer_get_bool (const gchar *path, gboolean def)
 }
 
 
-void gviewer_free_string_history (GList *strings)
-{
-    GList *temp;
-
-    for (temp = strings; temp; temp = temp->next)
-        if (temp->data!=NULL)
-        {
-            g_free (temp->data);
-            temp->data = NULL;
-        }
-    g_list_free(strings);
-}
-
-
-void gviewer_write_string_history (gchar *format, GList *strings)
-{
-    gchar key[128];
-
-    for (gint i=0; strings; strings = strings->next, ++i)
-    {
-        snprintf (key, sizeof (key), format, i);
-        gnome_config_set_string(key, (gchar *) strings->data);
-    }
-}
-
-
-gboolean gviewer_find_string_history (GList *strings, const gchar *text)
-{
-    for (; strings; strings = strings->next)
-        if (strings->data!=NULL)
-            if (strcmp((gchar *) strings->data, text)==0)
-                return TRUE;
-    return FALSE;
-}
-
-
-GList *gviewer_load_string_history (gchar *format, gint size)
-{
-    GList *list = NULL;
-
-    for (gint i=0; i < size || size == -1; ++i)
-    {
-        gchar *key = g_strdup_printf (format, i);
-        gchar *value = gviewer_get_string (key, NULL);
-        g_free (key);
-        if (!value)
-            break;
-        list = g_list_append (list, value);
-    }
-
-    return list;
-}
-
-
 int unicode2utf8 (unsigned int unicode, unsigned char *out)
 {
     int bytes_needed = 0;
@@ -224,7 +172,7 @@ guint8 *text2hex (const gchar *text, /*out*/ guint *buflen)
             {
                 idx++;
                 len++;
-            } 
+            }
             else
                 return NULL;
 
