@@ -1,6 +1,7 @@
 /*
     GNOME Commander - A GNOME based file manager
     Copyright (C) 2001-2006 Marcus Bjurman
+    Copyright (C) 2007-2009 Piotr Eljasiak
 
     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -22,7 +23,7 @@
 
 #include <gmodule.h>
 
-typedef struct
+struct PluginData
 {
     gboolean active;
     gboolean loaded;
@@ -33,12 +34,12 @@ typedef struct
     PluginInfo *info;
     GtkWidget *menu;
     GModule *module;
-} PluginData;
+};
 
 
-void plugin_manager_init (void);
-void plugin_manager_shutdown (void);
-GList *plugin_manager_get_all (void);
-void plugin_manager_show (void);
+void plugin_manager_init ();
+void plugin_manager_shutdown ();
+GList *plugin_manager_get_all ();
+void plugin_manager_show ();
 
 #endif // __PLUGIN_MANAGER_H__
